package org.cinco.generator.plugin.jobs;

import java.util.HashMap;
import java.util.Map;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;

public class GenerateCommonJavaJob extends Job {

	public GenerateCommonJavaJob(String name) {
		super(name);
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		try {
			String ecosystemName = (String) this.getProperty(new QualifiedName(
					null, RepositoryGeneratorConstants.PROPERTY_ECOSYSTEMNAME));
			String commonProjectName = (String) this
					.getProperty(new QualifiedName(
							null,
							RepositoryGeneratorConstants.PROPERTY_COMMONPROJECTNAME));
			String pkgPrefix = (String) this.getProperty(new QualifiedName(
					null, RepositoryGeneratorConstants.PROPERTY_PKGPREFIX));
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			IProject commonProject = workspace.getRoot().getProject(
					commonProjectName);
			if (!commonProject.exists()) {
				System.err.println("Common repository project "
						+ commonProjectName + " does not exist!");
				return Status.CANCEL_STATUS;
			}

			// Create the Ant runner, set properties and run with the build.xml
			// script
			IFile antFile = commonProject.getFile("build.xml");
			AntRunner runner = new AntRunner();
			Map<String, String> userProps = new HashMap<String, String>();
			userProps.put("ecosystemName", ecosystemName);
			userProps.put("projectURI", commonProject.getLocationURI()
					.toString());
			userProps.put("projectPkg", commonProjectName);
			userProps.put("pluginURI", "http://cinco.org/" + ecosystemName);
			userProps.put("pkgPrefix", pkgPrefix);
			// Now create the knowledge management model (from the UML file
			// first
			// transformed to ecosystem models)
			runner.setBuildFileLocation(antFile.getLocation().toOSString());
			runner.setArguments("-Dmessage=Building -verbose");
			runner.addUserProperties(userProps);
			runner.setExecutionTargets(new String[] { "container2Java" });
			
			runner.run(new SubProgressMonitor(monitor, 100));
		
		} catch (Exception e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}

}
