package org.cinco.generator.plugin.jobs;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.ant.core.IAntPropertyValueProvider;
import org.eclipse.ant.internal.core.IAntCoreConstants;
import org.eclipse.ant.internal.launching.IAntLaunchingPreferenceConstants;
import org.eclipse.ant.internal.launching.debug.IAntDebugConstants;
import org.eclipse.ant.internal.launching.debug.IAntDebugController;
import org.eclipse.ant.launching.IAntLaunchConstants;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.core.externaltools.internal.IExternalToolConstants;
import org.eclipse.uml2.uml.Model;
import org.osgi.framework.Bundle;

public class GenerateCommonProjectJob extends Job {

	public GenerateCommonProjectJob(String name) {
		super(name);
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		try {
			String commonProjectName = (String) this
					.getProperty(new QualifiedName(
							null,
							RepositoryGeneratorConstants.PROPERTY_COMMONPROJECTNAME));
			String ecosystemName = (String) this.getProperty(new QualifiedName(
					null, RepositoryGeneratorConstants.PROPERTY_ECOSYSTEMNAME));
			String pkgPrefix = (String) this.getProperty(new QualifiedName(
					null, RepositoryGeneratorConstants.PROPERTY_PKGPREFIX));
			IFile umlFile = (IFile) this.getProperty(new QualifiedName(null,
					RepositoryGeneratorConstants.PROPERTY_UMLMODEL));

			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			IProject commonProject = workspace.getRoot().getProject(
					commonProjectName);

			if (!commonProject.exists()) {
				commonProject.create(new SubProgressMonitor(monitor, 5));
			}
			commonProject.open(new SubProgressMonitor(monitor, 5));
			
			GeneratorHelper.createProjectStructure(new SubProgressMonitor(monitor, 5), commonProject, commonProjectName, false, null);

			// Create ecore models representing repository itemtypes and
			// container
			// in the "common" project

			// First create the build.xml file to the common project which
			// "does the job"
			Bundle bundle = Platform.getBundle("org.cinco.generator.plugin");
			Path path = new Path("scripts/build-common.xml");
			InputStream buildStream = FileLocator.openStream(bundle, path,
					false);
			IFile antFile = commonProject.getFile("build.xml");
			if (!antFile.exists()) {
				antFile.create(buildStream, true, new SubProgressMonitor(
						monitor, 5));
			}
			// Create the technical space description
			path = new Path("model/default-ts.xmi");
			InputStream tsStream = FileLocator.openStream(bundle, path, false);
			IFolder modelFolder = commonProject.getFolder("model");
			if (!modelFolder.exists()) {
				modelFolder.create(true, true, new SubProgressMonitor(monitor,
						5));
			}
			IFile tsFile = modelFolder.getFile("default-ts.xmi");
			if (!tsFile.exists()) {
				tsFile.create(tsStream, true,
						new SubProgressMonitor(monitor, 5));
			}

			// Copy the UML file to the model folder
			IFile umlOutput = modelFolder.getFile("ecosystem.uml");
			if(!umlOutput.exists()) {
				umlOutput.create(umlFile.getContents(), true, monitor);
			}
			
			String pkgPath = "src/" + commonProjectName.replace('.', '/');
			IFolder targetFolder = commonProject.getFolder(pkgPath);
			if(!targetFolder.exists()) {
				targetFolder.create(true, true, new SubProgressMonitor(monitor, 5));
			}
			
			/**
			 * Create a lauch configuration for the AntRunner.
			 */ 
			Map<String, String> userProps = new HashMap<String, String>();
			userProps.put("ecosystemName", ecosystemName);
			userProps.put("projectURI", commonProject.getLocationURI()
					.toString());
			userProps.put("projectPkg", commonProjectName);
			userProps.put("pluginURI", "http://cinco.org/" + ecosystemName.toLowerCase());
			userProps.put("umlModelURI", "model/ecosystem.uml"); // umlFile.getLocationURI().toString());
			userProps.put("pkgPrefix", pkgPrefix);
			System.out.println(userProps);
			
//			ILaunchManager lm = DebugPlugin.getDefault().getLaunchManager();
//			ILaunchConfigurationType lt = 
//				lm.getLaunchConfigurationType(IAntLaunchConstants.ID_ANT_BUILDER_LAUNCH_CONFIGURATION_TYPE);
//			String lcName = lm.generateLaunchConfigurationName(ecosystemName);
//			ILaunchConfigurationWorkingCopy wc =
//				lt.newInstance(null, lcName);
//			wc.setAttribute(IAntLaunchConstants.ATTR_ANT_PROPERTIES, userProps);
//			wc.setAttribute(IAntLaunchConstants.ATTR_ANT_TARGETS, "Ecosystem2KM");
//			wc.setAttribute(DebugPlugin.ATTR_CAPTURE_OUTPUT, true);
//			wc.setAttribute(IExternalToolConstants.ATTR_LOCATION, antFile.getLocation().toOSString());
//			wc.launch(ILaunchManager.RUN_MODE, new SubProgressMonitor(monitor, 10));

			// Create the Ant runner, set properties and run with the build.xml
			// script
			// Now create the knowledge management model (from the UML file
			// first transformed to ecosystem models)
			AntRunner runner = new AntRunner();
			runner.setBuildFileLocation(antFile.getLocation().toOSString());
			runner.setArguments("-Dmessage=Building -verbose");
			runner.addUserProperties(userProps);
//			runner.setExecutionTargets(new String[] { "Ecosystem2KM" });
//			runner.run(new SubProgressMonitor(monitor, 100));

			// Create the Ant runner, set properties and run with the build.xml
			// script
//			AntRunner runner = new AntRunner();
//			Map<String, String> commonUserProps = new HashMap<String, String>();
//			commonUserProps.put("ecosystemName", ecosystemName);
//			commonUserProps.put("projectURI", commonProject.getLocationURI()
//					.toString());
//			commonUserProps.put("projectPkg", commonProjectName);
//			commonUserProps.put("pluginURI", "http://cinco.org/" + ecosystemName);
//			commonUserProps.put("pkgPrefix", pkgPrefix);
//			// Now create the knowledge management model (from the UML file
//			// first
//			// transformed to ecosystem models)
//			runner.setBuildFileLocation(antFile.getLocation().toOSString());
//			runner.setArguments("-Dmessage=Building -verbose");
//			runner.addUserProperties(commonUserProps);
			runner.setExecutionTargets(new String[] { "container2Java" });
			
			runner.run(new SubProgressMonitor(monitor, 100));
		
		} catch (IOException e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		} catch (CoreException e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}

}
