package org.cinco.generator.plugin.jobs;

import java.util.ArrayList;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.cinco.metamodel.ecosystem.knowledgemanagementmodel.repositories.KnowledgeRepository;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.URI;
import org.eclipse.gmt.modisco.java.generation.files.Generate_JavaStructures;

public class GenerateRepoJavaJob extends Job {

	private String ecosystemName;
	private String commonProjectName;
	private String pkgPrefix;
	private KnowledgeRepository repo;
	private IProject commonProject;
	private String repoProjectName;
	private IProject repoProject;

	public GenerateRepoJavaJob(String name) {
		super(name);
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		try {
		ecosystemName = (String) this.getProperty(new QualifiedName(null, RepositoryGeneratorConstants.PROPERTY_ECOSYSTEMNAME));
		commonProjectName = (String) this.getProperty(new QualifiedName(null, RepositoryGeneratorConstants.PROPERTY_COMMONPROJECTNAME));
		pkgPrefix = (String) this.getProperty(new QualifiedName(null, RepositoryGeneratorConstants.PROPERTY_PKGPREFIX));
		repo = (KnowledgeRepository) this.getProperty(new QualifiedName(null, RepositoryGeneratorConstants.PROPERTY_KNOWLEDGEREPOSITORY));
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		commonProject = workspace.getRoot().getProject(commonProjectName);

		repoProjectName = pkgPrefix + ecosystemName.toLowerCase()
		+ ".repository." + repo.getName().toLowerCase();

		repoProject = workspace.getRoot().getProject(repoProjectName);
		
		IFile javaModel = repoProject.getFile("model/modisco-java.xmi");
		if(!javaModel.exists()) {
			System.err.println(repoProject.getName() + "/model/modisco-java.xmi is missing!");
			return Status.CANCEL_STATUS;
		}
		IFolder javaSrc = repoProject.getFolder("src"); 
		if(!javaSrc.exists()) {
			javaSrc.create(true, true, monitor);
		}
		
		Generate_JavaStructures javaGenerator;
		System.out.println("Generating Java code from "
				+ javaModel.getLocation() + " to " + javaSrc.getLocation());
		java.io.File srcDir = new java.io.File(javaSrc.getFullPath().toOSString());
		javaGenerator = new Generate_JavaStructures(
				URI.createFileURI(javaModel.getLocation()
						.toOSString()), 
						srcDir,
				new ArrayList<Object>());
		
		javaGenerator.doGenerate(null);
		} catch(Exception e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}

}
