package org.cinco.generator.plugin.jobs;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.cinco.metamodel.ecosystem.knowledgemanagementmodel.repositories.KnowledgeRepository;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.URI;
import org.eclipse.gmt.modisco.java.generation.files.Generate_JavaStructures;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.launching.JavaRuntime;
import org.osgi.framework.Bundle;

public class GenerateRepositoryJob extends Job {

	private String ecosystemName;
	private String commonProjectName;
	private String pkgPrefix;
	private KnowledgeRepository repo;
	private IProject commonProject;

	public GenerateRepositoryJob(String name) {
		super(name);
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		ecosystemName = (String) this.getProperty(new QualifiedName(null,
				RepositoryGeneratorConstants.PROPERTY_ECOSYSTEMNAME));
		commonProjectName = (String) this.getProperty(new QualifiedName(null,
				RepositoryGeneratorConstants.PROPERTY_COMMONPROJECTNAME));
		pkgPrefix = (String) this.getProperty(new QualifiedName(null,
				RepositoryGeneratorConstants.PROPERTY_PKGPREFIX));
		repo = (KnowledgeRepository) this.getProperty(new QualifiedName(null,
				RepositoryGeneratorConstants.PROPERTY_KNOWLEDGEREPOSITORY));
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		commonProject = workspace.getRoot().getProject(commonProjectName);
		String projectName = pkgPrefix + ecosystemName.toLowerCase()
				+ ".repository." + repo.getName().toLowerCase();

		IProject repoProject = workspace.getRoot().getProject(projectName);

		// Create the repository Ant -file
		IFile antFile = repoProject.getFile("build.xml");
		Bundle bundle = Platform.getBundle("org.cinco.generator.plugin");
		Path path = new Path("scripts/build-repository.xml");
		try {
			InputStream buildStream = FileLocator.openStream(bundle, path,
					false);
			if (!antFile.exists()) {
				antFile.create(buildStream, true, new SubProgressMonitor(
						monitor, 5));
			}
		} catch (Exception e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}

		/*
		 * Do: 1) repository model generation (ItemType, Container) 2) genModel
		 * generation 3) code generation from the genModel
		 */

		// Task: run the Ant script
		runAntScript(repo, new SubProgressMonitor(monitor, 5));

		// Create eclipse projects (OSGI, SCA)
//		createRepositoryProject(repo, monitor);

		return Status.OK_STATUS;
	}

	private void runAntScript(KnowledgeRepository r, IProgressMonitor monitor) {
		try {
			String projectName = pkgPrefix + ecosystemName.toLowerCase()
					+ ".repository." + r.getName().toLowerCase();

			IWorkspace workspace = ResourcesPlugin.getWorkspace();

			IProject repoProject = workspace.getRoot().getProject(projectName);
			if (!repoProject.exists()) {
				repoProject.create(new SubProgressMonitor(monitor, 5));
			}
			repoProject.open(new SubProgressMonitor(monitor, 5));

			// Set project dependencies
			IProjectDescription desc = repoProject.getDescription();
			desc.setReferencedProjects(new IProject[] { commonProject });
			repoProject.setDescription(desc, new NullProgressMonitor());

			Map<String, String> userProps = new HashMap<String, String>();
			userProps.put("commonProjectName", commonProjectName);
			userProps.put("ecosystemName", ecosystemName);
			userProps.put("projectURI", commonProject.getLocationURI()
					.toString());
			userProps.put("projectPkg", commonProjectName);
			userProps.put("repoProjectURI", repoProject.getLocationURI()
					.toString());
			userProps.put("pluginURI",
					"http://cinco.org/" + ecosystemName.toLowerCase());
			userProps.put("repositoryName", r.getName());
			userProps.put("pkgPrefix", pkgPrefix);
			System.out.println("Running KM2RepositoryPSM with props: "
					+ userProps);
			AntRunner runner = new AntRunner();

			IFile antFile = repoProject.getFile("build.xml");

			runner.setBuildFileLocation(antFile.getLocation().toOSString());
			runner.setArguments("-Dmessage=Building -verbose");
			runner.addUserProperties(userProps);
			runner.setExecutionTargets(new String[] { "RepositoryJava" });
			
			if(AntRunner.isBuildRunning()) {
				System.out.println("sleeping..");
				this.schedule(5000);
			}
			runner.run(new SubProgressMonitor(monitor, 5));

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Creates a knowledge repository project for the knowledge repository
	 * 
	 * @param r
	 *            The knowledge repository
	 * @param subMon1
	 */
	private void createRepositoryProject(KnowledgeRepository r,
			IProgressMonitor monitor) {
		IProject project = null;
		IFile antFile = null;
		try {
			String projectName = pkgPrefix + ecosystemName.toLowerCase()
					+ ".repository." + r.getName().toLowerCase();

			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			project = workspace.getRoot().getProject(projectName);
			if (!project.exists()) {
				project.create(new SubProgressMonitor(monitor, 5));
			}
			project.open(new NullProgressMonitor());
			System.out.println("Project " + project.getName() + " created!");

//			antFile = project.getFile("build.xml");
//			Job.getJobManager().beginRule(antFile, monitor);

			// Create & set the 'bin' folder
			IFolder binFolder = project.getFolder("bin");
			if (!binFolder.exists()) {
				binFolder.create(false, true,
						new SubProgressMonitor(monitor, 5));
			}

			// Create the 'src' folder
			IFolder sourceFolder = project.getFolder("src");
			if (!sourceFolder.exists()) {
				sourceFolder.create(false, true, new SubProgressMonitor(
						monitor, 5));
			}
			createProjectStructure(monitor, project, projectName, binFolder,
					sourceFolder);

			IFile javaModel = project.getFile("model/modisco-java.xmi");
			if (!javaModel.exists()) {
				System.err.println(project.getName()
						+ "/model/modisco-java.xmi is missing!");
				return;
			}
			Generate_JavaStructures javaGenerator;
			System.out.println("Generating Java code from "
					+ javaModel.getLocation() + " to "
					+ sourceFolder.getLocation());
			java.io.File srcDir = new java.io.File(sourceFolder.getFullPath()
					.toOSString());
			javaGenerator = new Generate_JavaStructures(
					URI.createFileURI(javaModel.getLocation().toOSString()),
					srcDir, new ArrayList<Object>());

			javaGenerator.doGenerate(null);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
//			Job.getJobManager().endRule(antFile);
		}
	}

	private void createProjectStructure(IProgressMonitor monitor,
			IProject project, String projectName, IFolder binFolder,
			IFolder sourceFolder) throws CoreException, JavaModelException {
		addProjectNature(monitor, project, JavaCore.NATURE_ID);

		IJavaProject javaProject = JavaCore.create(project);

		javaProject.setOutputLocation(binFolder.getFullPath(),
				new SubProgressMonitor(monitor, 5));

		// Set default classpath entries
		IPackageFragmentRoot root = javaProject
				.getPackageFragmentRoot(sourceFolder);
		Set<IClasspathEntry> entries = new HashSet<IClasspathEntry>();
		entries.addAll(Arrays.asList(javaProject.getRawClasspath()));

		// Remove old CPE_SOURCE entries!
		Set<IClasspathEntry> removeEntries = new HashSet<IClasspathEntry>();
		for (IClasspathEntry e : entries) {
			if (e.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
				removeEntries.add(e);
			}
		}
		entries.removeAll(removeEntries);

		// Set new CPE_SOURCE entries
		entries.add(JavaCore.newSourceEntry(root.getPath()));

		// Set default JRE container entry
		entries.add(JavaRuntime.getDefaultJREContainerEntry());

		javaProject.setRawClasspath(
				entries.toArray(new IClasspathEntry[entries.size()]),
				new SubProgressMonitor(monitor, 5));

		// Create META-INF and OSGI-INF folders
		IFolder miFolder = project.getFolder("/META-INF");
		if (!miFolder.exists()) {
			miFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}
		IFolder oiFolder = project.getFolder("/OSGI-INF");
		if (!oiFolder.exists()) {
			oiFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}

		// Create model folder
		IFolder mFolder = project.getFolder("/model");
		if (!mFolder.exists()) {
			mFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}

		// Add PDE (OSGI) nature
		addProjectNature(monitor, project, "org.eclipse.pde.PluginNature");

		// Add SCA nature
		addProjectNature(monitor, project,
				"org.eclipse.stp.sca.common.scaNature");

		IFile mfFile = miFolder.getFile("MANIFEST.MF");

		/*
		 * IFile antSrc = project.getFile("build.xml");
		 */

		// First refresh the project!
		project.refreshLocal(IResource.DEPTH_INFINITE,
				new NullProgressMonitor());

		String manifest = "Manifest-Version: 1.0\n"
				+ "Bundle-ManifestVersion: 2\n"
				+ "Bundle-Name: "
				+ projectName
				+ "\n"
				+ "Bundle-SymbolicName: "
				+ projectName
				+ "\n"
				+ "Bundle-Version: 1.0.0\n"
				+ "Bundle-ClassPath: .\n"
				+ "Bundle-Vendor: Toni Ruokolainen\n"
				// + "Bundle-Localization: plugin\n"
				+ "Bundle-RequiredExecutionEnvironment: JavaSE-1.6\n" // Export-Package
																		// -declarations
																		// should
																		// go
																		// here!!
				+ "Require-Bundle: org.eclipse.core.runtime,\n"
				+ " org.eclipse.emf.ecore;visibility:=reexport,\n"
				+ " org.eclipse.emf.ecore.xmi;visibility:=reexport\n"
				+ "Bundle-ActivationPolicy: lazy\n";

		ByteArrayInputStream baout = new ByteArrayInputStream(
				manifest.getBytes());
		mfFile.delete(true, new NullProgressMonitor());
		mfFile.create(baout, true, new SubProgressMonitor(monitor, 5));
	}

	private void addProjectNature(IProgressMonitor monitor, IProject project,
			String natureId) throws CoreException {
		// Add Java nature to the project
		IProjectDescription prDesc = project.getDescription();
		String[] prNatureIds = prDesc.getNatureIds();
		String[] newNatureIds = new String[prNatureIds.length + 1];
		System.arraycopy(prNatureIds, 0, newNatureIds, 0, prNatureIds.length);
		newNatureIds[prNatureIds.length] = natureId;
		prDesc.setNatureIds(newNatureIds);
		project.setDescription(prDesc, new SubProgressMonitor(monitor, 5));
	}

}
