package org.cinco.generator.plugin.jobs;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.launching.JavaRuntime;

public class GeneratorHelper {

	static public void createProjectStructure(IProgressMonitor monitor,
			IProject project, String projectName, boolean createManifest, ArrayList<String> requiredBundles) throws CoreException, JavaModelException {
		addProjectNature(monitor, project, JavaCore.NATURE_ID);

		IFolder binFolder = project.getFolder("bin");
		if(!binFolder.exists()) {
			binFolder.create(true, true, monitor);
		}
		IFolder sourceFolder = project.getFolder("src");
		if(!sourceFolder.exists()) {
			sourceFolder.create(true, true, monitor);
		}

		IJavaProject javaProject = JavaCore.create(project);
		javaProject.setOutputLocation(binFolder.getFullPath(),
				new SubProgressMonitor(monitor, 5));

		/*
		String pkgPath = projectName.replace('.', '/');
		IFolder parent = project.getFolder("src");
		IFolder pkgRoot = null;
		for(String s : pkgPath.split(pkgPath, '/')) {
			if(!parent.exists()) {
				parent.create(true, true, null);
			}
			parent = parent.getFolder(s);
			if(pkgRoot == null) {
				pkgRoot = parent;
			}
		}
		*/
//		IFolder targetFolder = project.getFolder(pkgPath);
//		if(!targetFolder.exists()) {
//			targetFolder.create(true, true, new SubProgressMonitor(monitor, 5));
//		}
		
		setClasspathEntries(monitor, project);
		
		// Add the default package
		
//		root.createPackageFragment(projectName, true, new SubProgressMonitor(monitor, 5));
		
		// Create META-INF and OSGI-INF folders
		IFolder miFolder = project.getFolder("/META-INF");
		if (!miFolder.exists()) {
			miFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}
		IFolder oiFolder = project.getFolder("/OSGI-INF");
		if (!oiFolder.exists()) {
			oiFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}
		// Create model folder
		IFolder mFolder = project.getFolder("/model");
		if (!mFolder.exists()) {
			mFolder.create(false, true, new SubProgressMonitor(monitor, 5));
		}

		// Refresh the project!
		project.refreshLocal(IResource.DEPTH_INFINITE,
				new NullProgressMonitor());
	
	}

	private static void addProjectNature(IProgressMonitor monitor, IProject project,
			String natureId) throws CoreException {

		IProjectDescription prDesc = project.getDescription();
		String[] prNatureIds = prDesc.getNatureIds();
		String[] newNatureIds = new String[prNatureIds.length + 1];
		System.arraycopy(prNatureIds, 0, newNatureIds, 0, prNatureIds.length);
		newNatureIds[prNatureIds.length] = natureId;
		prDesc.setNatureIds(newNatureIds);
		project.setDescription(prDesc, new SubProgressMonitor(monitor, 5));
	}

	public static void addNatures(IProgressMonitor monitor, IProject project) throws CoreException {
		// Add PDE (OSGI) nature
		addProjectNature(monitor, project, "org.eclipse.pde.PluginNature");
		
		// Add SCA nature
		addProjectNature(monitor, project,
				"org.eclipse.stp.sca.common.scaNature");
	}

	public static void setProjectBuilders(IProgressMonitor monitor, IProject project) throws CoreException {
		String [] builderNames = new String [] {
			"org.eclipse.jdt.core.javabuilder",
			"org.eclipse.pde.ManifestBuilder",
			"org.eclipse.pde.SchemaBuilder",
			"org.eclipse.pde.ds.core.builder"
		};
		IProjectDescription projectDesc = project.getDescription();
		ICommand[] builders = projectDesc.getBuildSpec();
		if(builders == null) {
			builders = new ICommand [0];
		}
		for(String n : builderNames) {
			boolean hasBuilder = false;
			for(int i = 0; i < builders.length; ++i) {
				if(n.equals(builders[i].getBuilderName())) {
					hasBuilder = true;
					break;
				}
			}
			if(!hasBuilder) {
				ICommand [] oldBuilders = builders;
				builders = new ICommand [oldBuilders.length + 1];
				System.arraycopy(oldBuilders, 0, builders, 0, oldBuilders.length);
				builders[oldBuilders.length] = projectDesc.newCommand();
				builders[oldBuilders.length].setBuilderName(n);
			}
		}
		projectDesc.setBuildSpec(builders);
		project.setDescription(projectDesc, new SubProgressMonitor(monitor, 1));		
	}
	
	public static void setClasspathEntries(IProgressMonitor monitor, IProject project) throws CoreException {
		IJavaProject javaProject = JavaCore.create(project);

		IFolder sourceFolder = project.getFolder("src");
		if(!sourceFolder.exists()) {
			sourceFolder.create(true, true, monitor);
		}
		
		IPackageFragmentRoot root = javaProject.getPackageFragmentRoot(sourceFolder);
	
		// Add the default package
		root.createPackageFragment(project.getName(), true, new SubProgressMonitor(monitor, 5));
		
		// Set default classpath entries
		Set<IClasspathEntry> entries = new HashSet<IClasspathEntry>();
		entries.addAll(Arrays.asList(javaProject.getRawClasspath()));

		// Remove old CPE_SOURCE entries!
		Set<IClasspathEntry> removeEntries = new HashSet<IClasspathEntry>();
		for (IClasspathEntry e : entries) {
			if (e.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
				removeEntries.add(e);
			}
		}
		entries.removeAll(removeEntries);

		// Set new CPE_SOURCE entries
		entries.add(JavaCore.newSourceEntry(root.getPath()));

		// Set default JRE container entry
		entries.add(JavaRuntime.getDefaultJREContainerEntry());

		// Set PDE entries
		entries.add(JavaCore.newContainerEntry(new Path("org.eclipse.pde.core.requiredPlugins")));
				
		javaProject.setRawClasspath(
				entries.toArray(new IClasspathEntry[entries.size()]),
				new SubProgressMonitor(monitor, 5));
	}
}
