package org.cinco.generator.plugin.jobs;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.cinco.generator.plugin.preferences.PreferenceConstants;
import org.cinco.metamodel.ecosystem.ecosystemmodel.EcosystemModel;
import org.cinco.metamodel.ecosystem.foundations.models.ModelElement;
import org.cinco.metamodel.ecosystem.knowledgemanagementmodel.repositories.KnowledgeRepository;
import org.cinco.metamodel.ecosystem.seadescription.SEADescription;
import org.cinco.metamodel.ecosystem.seadescription.SEAModel;
import org.cinco.metamodel.ecosystem.seadescription.SeadescriptionPackage;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.preferences.IPreferencesService;
import org.eclipse.emf.codegen.ecore.generator.Generator;
import org.eclipse.emf.codegen.ecore.generator.GeneratorAdapterFactory;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage;
import org.eclipse.emf.codegen.ecore.genmodel.generator.GenBaseGeneratorAdapter;
import org.eclipse.emf.codegen.ecore.genmodel.generator.GenModelGeneratorAdapterFactory;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UML212UMLResource;
import org.osgi.framework.Bundle;

public class SEAM2JavaJob extends Job {

	private String pkgPrefix;
	private IFile seamFile;
	private Resource seamResource;
	private String ecosystemName;
	private String commonProjectName;
	private IProject commonProject;

	public SEAM2JavaJob(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}

	@SuppressWarnings("unchecked")
	@Override
	protected IStatus run(IProgressMonitor monitor) {
		// Read the properties common for all jobs.
		monitor.beginTask("Create Java code from SEAM", 1000);

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		String workspaceDir = workspace.getRoot().getLocation().toFile()
				.getPath().toString();
		String eclipseDir = System.getProperty("eclipse.launcher");
		System.out.println("Eclipse directory: " + eclipseDir);
		System.out.println("Workspace directory: " + workspaceDir);

		IPreferencesService prefService = Platform.getPreferencesService();
		pkgPrefix = prefService.getString("org.cinco.generator.plugin",
				PreferenceConstants.P_PKGPREFIX, null, null);
		if (pkgPrefix == null) {
			pkgPrefix = "org.cinco";
		}
		// The pkgPrefix is assumed in the following code to end
		// with a dot ".".
		if ((pkgPrefix.length() > 1) && (!pkgPrefix.endsWith("."))) {
			pkgPrefix = pkgPrefix + ".";
		}

		/**
		 * Task 1: Read the SEAM model.
		 */
		ResourceSet seamResourceSet = new ResourceSetImpl();
		seamResourceSet.getPackageRegistry().put(SeadescriptionPackage.eNS_URI,
				SeadescriptionPackage.eINSTANCE);
		seamResourceSet.getPackageRegistry().put(
				UML212UMLResource.UML_METAMODEL_NS_URI, UMLPackage.eINSTANCE);

		QualifiedName seamFilekey = new QualifiedName(null,
				RepositoryGeneratorConstants.SEAM_IFILE);
		seamFile = (IFile) this.getProperty(seamFilekey);

		URI seamURI = URI.createPlatformResourceURI(seamFile.getFullPath()
				.toString(), true);

		seamResource = seamResourceSet.getResource(seamURI, true);
		if (seamResource == null) {
			System.err.println("No SEAM file found from " + seamURI);
			return Status.CANCEL_STATUS;
		}
		;
		System.out
				.println("Loaded SEAM model from URI " + seamResource.getURI());

		if (!(seamResource.getContents().get(0) instanceof SEADescription)) {
			System.err.println("Not a service ecosystem architecture model! "
					+ seamResource.getURI());
			return Status.CANCEL_STATUS;
		}

		SEADescription model = (SEADescription) seamResource.getContents().get(
				0);
		for (SEAModel m : (List<SEAModel>) model.getElements()) {
			if (m instanceof EcosystemModel) {
				ecosystemName = ((EcosystemModel) m).getName();
				break;
			}
		}
		if (ecosystemName == null) {
			System.err.println("No ecosystem model found from "
					+ seamResource.getURI());
			return Status.CANCEL_STATUS;
		}

		System.out.println("Ecosystem name = " + ecosystemName);
		commonProjectName = pkgPrefix + ecosystemName.toLowerCase() + ".common";
		commonProject = workspace.getRoot().getProject(commonProjectName);
		assert (commonProject != null);
		monitor.worked(100);

		List<IProject> repoProjects = new ArrayList<IProject>();
		// 2.1: Get all knowledge repository elements from the SeaDescription
		TreeIterator<EObject> tr = model.eAllContents();
		List<KnowledgeRepository> repoList = new ArrayList<KnowledgeRepository>();
		while (tr.hasNext()) {
			EObject obj = tr.next();
			if (obj instanceof KnowledgeRepository) {
				repoList.add((KnowledgeRepository) obj);
			}
		}

		// 2.2: Iterate through all KnowledgeRepository elements and retrieve
		// corresponding Eclipse projects.
		for (ModelElement r : repoList) {
			try {
				if (r instanceof KnowledgeRepository) {
					KnowledgeRepository repo = (KnowledgeRepository) r;
					String repoProjectName = pkgPrefix
							+ ecosystemName.toLowerCase() + "."
							+ repo.getName().toLowerCase();
					IProject repoProject = workspace.getRoot().getProject(
							repoProjectName);
					repoProjects.add(repoProject);
					repoProject.open(new NullProgressMonitor());
				}
			} catch (CoreException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			}
		}

		// Check if there are any repositories in the model!
		if (repoProjects.size() < 1) {
			return Status.CANCEL_STATUS;
		}

		/**
		 * Task 5: Lauch SEAM2PSM M2T transformations.
		 */
		
		// 5.1 SEAM2Java
		for(IProject project : repoProjects) {
			try {
				project.refreshLocal(IResource.DEPTH_INFINITE, null);
			} catch (CoreException e1) {
				e1.printStackTrace();
				return Status.CANCEL_STATUS;
			}
			System.out.println("Launching SEAM2Java for project " + project.getName());
			URI modelURI = URI.createFileURI(seamFile.getLocation().toString());
			String destFolderName = project.getName().replace('.', '/');
			IFolder folder = project.getFolder("src/" + destFolderName);
			IFolder target = folder.getFolder("service");
			if(!target.exists()) {
				try {
					target.create(true, true, monitor);
				} catch (CoreException e) {
					e.printStackTrace();
					return Status.CANCEL_STATUS;
				}
			}
			IFile propFile = project.getFile("repository.properties");
			System.out.println("Using repository properties from location " + propFile.getLocation().toOSString());
			try {
				org.cinco.m2t.seam2java.main.Seam2Java generator =
					new org.cinco.m2t.seam2java.main.Seam2Java(modelURI, target.getLocation().toFile(), new ArrayList<Object>());
				generator.addPropertiesFile(propFile.getLocation().toOSString());
				generator.doGenerate(new BasicMonitor());
			} catch (IOException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			}
		}
				
		// 5.1 SEAM2Manifest
		for(IProject project : repoProjects) {
			try {
				project.refreshLocal(IResource.DEPTH_INFINITE, null);
			} catch (CoreException e1) {
				e1.printStackTrace();
				return Status.CANCEL_STATUS;
			}
			System.out.println("Launching SEAM2Manifest for project " + project.getName());
			URI modelURI = URI.createFileURI(seamFile.getLocation().toString());
			IFolder target = project.getFolder("META-INF");
			if(!target.exists()) {
				try {
					target.create(true, true, monitor);
				} catch (CoreException e) {
					e.printStackTrace();
					return Status.CANCEL_STATUS;
				}
			}
			IFile propFile = project.getFile("repository.properties");
			System.out.println("Using repository properties from location " + propFile.getLocation().toOSString());
			try {
				org.cinco.m2t.seam2manifest.main.Seam2manifest generator = 
					new org.cinco.m2t.seam2manifest.main.Seam2manifest(modelURI, target.getLocation().toFile(), new ArrayList<Object>());
				generator.addPropertiesFile(propFile.getLocation().toOSString());
				generator.doGenerate(new BasicMonitor());
			} catch (IOException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			}
		}
		
		return Status.OK_STATUS;
	}

	private void generateJava(GenModel genModel) {
		GeneratorAdapterFactory.Descriptor.Registry.INSTANCE.addDescriptor(
				GenModelPackage.eNS_URI,
				GenModelGeneratorAdapterFactory.DESCRIPTOR);

		// Create the generator and set the model-level input object.
		//
		Generator generator = new Generator();
		generator.setInput(genModel);
		// Generator model code.
		//
		genModel.setForceOverwrite(true);
		genModel.reconcile();
		genModel.setCanGenerate(true);
		genModel.setValidateModel(true);
		genModel.setUpdateClasspath(false);
		genModel.setCodeFormatting(true);
		generator.generate(genModel,
				GenBaseGeneratorAdapter.MODEL_PROJECT_TYPE,
				new BasicMonitor.Printing(System.out));
	}

	private void runAntforProjects(IProgressMonitor monitor,
			List<IProject> repoProjects, int monInc, String executionTarget)
			throws CoreException, InterruptedException {
		for (IProject repoProject : repoProjects) {
			System.out.println("Run Ant target " + executionTarget
					+ " for project " + repoProject.getName());
			IFile antFile = repoProject.getFile("build.xml");
			AntRunner runner = new AntRunner();

			// runner.addBuildLogger(className);
			runner.setBuildFileLocation(antFile.getLocation().toOSString());
			runner.setArguments("-Dmessage=Building -verbose");
			// runner.addUserProperties(userProps);
			runner.setExecutionTargets(new String[] { executionTarget });
			
			runner.run(new SubProgressMonitor(monitor, 10));
			while (AntRunner.isBuildRunning()) {
				this.wait(50);
			}
			monitor.worked(monInc);
			repoProject.refreshLocal(IResource.DEPTH_INFINITE, null);
		}

		while (AntRunner.isBuildRunning()) {
			try {
				this.wait(50);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	private void createFile(String fileName, Bundle bundle, IProject repoProject)
			throws IOException, CoreException {
		Path buildPropsPath = new Path(fileName);
		InputStream buildPropsStream = FileLocator.openStream(bundle,
				buildPropsPath, false);
		IFile buildPropFile = repoProject.getFile(fileName);
		if (!buildPropFile.exists()) {
			buildPropFile.create(buildPropsStream, true,
					new NullProgressMonitor());
		}
	}

}
