package org.cinco.generator.plugin.jobs;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.cinco.generator.plugin.preferences.PreferenceConstants;
import org.cinco.metamodel.ecosystem.ecosystemmodel.EcosystemModel;
import org.cinco.metamodel.ecosystem.foundations.models.ModelElement;
import org.cinco.metamodel.ecosystem.knowledgemanagementmodel.repositories.KnowledgeRepository;
import org.cinco.metamodel.ecosystem.seadescription.SEADescription;
import org.cinco.metamodel.ecosystem.seadescription.SEAModel;
import org.cinco.metamodel.ecosystem.seadescription.SeadescriptionPackage;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.preferences.IPreferencesService;
import org.eclipse.emf.codegen.ecore.generator.Generator;
import org.eclipse.emf.codegen.ecore.generator.GeneratorAdapterFactory;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage;
import org.eclipse.emf.codegen.ecore.genmodel.generator.GenBaseGeneratorAdapter;
import org.eclipse.emf.codegen.ecore.genmodel.generator.GenModelGeneratorAdapterFactory;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UML212UMLResource;
import org.osgi.framework.Bundle;

public class SEAM2PSMJob extends Job {

	private String pkgPrefix;
	private IFile seamFile;
	private Resource seamResource;
	private String ecosystemName;
	private String commonProjectName;
	private IProject commonProject;

	public SEAM2PSMJob(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}

	@SuppressWarnings("unchecked")
	@Override
	protected IStatus run(IProgressMonitor monitor) {
		// Read the properties common for all jobs.
		monitor.beginTask("Create Ecosystem Facilities", 1000);

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		String workspaceDir = workspace.getRoot().getLocation().toFile()
				.getPath().toString();
		String eclipseDir = System.getProperty("eclipse.launcher");
		System.out.println("Eclipse directory: " + eclipseDir);
		System.out.println("Workspace directory: " + workspaceDir);

		IPreferencesService prefService = Platform.getPreferencesService();
		pkgPrefix = prefService.getString("org.cinco.generator.plugin",
				PreferenceConstants.P_PKGPREFIX, null, null);
		if (pkgPrefix == null) {
			pkgPrefix = "org.cinco";
		}
		// The pkgPrefix is assumed in the following code to end
		// with a dot ".".
		if ((pkgPrefix.length() > 1) && (!pkgPrefix.endsWith("."))) {
			pkgPrefix = pkgPrefix + ".";
		}

		/**
		 * Task 1: Read the SEAM model.
		 */
		ResourceSet seamResourceSet = new ResourceSetImpl();
		seamResourceSet.getPackageRegistry().put(SeadescriptionPackage.eNS_URI,
				SeadescriptionPackage.eINSTANCE);
		seamResourceSet.getPackageRegistry().put(
				UML212UMLResource.UML_METAMODEL_NS_URI, UMLPackage.eINSTANCE);

		QualifiedName seamFilekey = new QualifiedName(null,
				RepositoryGeneratorConstants.SEAM_IFILE);
		seamFile = (IFile) this.getProperty(seamFilekey);

		URI seamURI = URI.createPlatformResourceURI(seamFile.getFullPath()
				.toString(), true);

		seamResource = seamResourceSet.getResource(seamURI, true);
		if (seamResource == null) {
			System.err.println("No SEAM file found from " + seamURI);
			return Status.CANCEL_STATUS;
		}
		;
		System.out
				.println("Loaded UML model from URI " + seamResource.getURI());

		if (!(seamResource.getContents().get(0) instanceof SEADescription)) {
			System.err.println("Not a service ecosystem architecture model! "
					+ seamResource.getURI());
			return Status.CANCEL_STATUS;
		}

		SEADescription model = (SEADescription) seamResource.getContents().get(
				0);
		for (SEAModel m : (List<SEAModel>) model.getElements()) {
			if (m instanceof EcosystemModel) {
				ecosystemName = ((EcosystemModel) m).getName();
				break;
			}
		}
		if (ecosystemName == null) {
			System.err.println("No ecosystem model found from "
					+ seamResource.getURI());
			return Status.CANCEL_STATUS;
		}

		System.out.println("Ecosystem name = " + ecosystemName);
		commonProjectName = pkgPrefix + ecosystemName.toLowerCase() + ".common";
		commonProject = workspace.getRoot().getProject(commonProjectName);
		assert (commonProject != null);
		monitor.worked(10);

		/**
		 * Task 2: Create repository projects.
		 */
		List<IProject> repoProjects = new ArrayList<IProject>();

		// 2.1: Get all knowledge repository elements from the SeaDescription
		TreeIterator<EObject> tr = model.eAllContents();
		List<KnowledgeRepository> repoList = new ArrayList<KnowledgeRepository>();
		while (tr.hasNext()) {
			EObject obj = tr.next();
			if (obj instanceof KnowledgeRepository) {
				repoList.add((KnowledgeRepository) obj);
			}
		}

		// 2.2: Iterate through all KnowledgeRepository elements and create
		// corresponding Eclipse projects.
		for (ModelElement r : repoList) {
			try {
				if (r instanceof KnowledgeRepository) {
					KnowledgeRepository repo = (KnowledgeRepository) r;
					String repoProjectName = pkgPrefix
							+ ecosystemName.toLowerCase() + "."
							+ repo.getName().toLowerCase();
					IProject repoProject = workspace.getRoot().getProject(
							repoProjectName);
					repoProjects.add(repoProject);
					if (!repoProject.exists()) {
						repoProject.create(new NullProgressMonitor());
					}
					repoProject.open(new NullProgressMonitor());

					// Create files
					Bundle bundle = Platform
							.getBundle("org.cinco.generator.plugin");
					Path antPath = new Path("scripts/build-repository.xml");
					InputStream buildStream = FileLocator.openStream(bundle,
							antPath, false);
					IFile antFile = repoProject.getFile("build.xml");
					if (!antFile.exists()) {
						antFile.create(buildStream, true,
								new SubProgressMonitor(monitor, 1));
					}
					/*
					 * this.createFile("build.properties", bundle, repoProject);
					 * this.createFile("plugin.properties", bundle,
					 * repoProject); this.createFile("plugin.xml", bundle,
					 * repoProject);
					 */
					System.out.println("commonProject: "
							+ commonProject.getName());
					System.out.println("commonProjectURI: "
							+ commonProject.getLocationURI().toString());

					// Set project properties
					Map<String, String> userProps = new HashMap<String, String>();
					userProps.put("commonProjectURI", commonProject
							.getLocationURI().toString());
					userProps.put("workspaceDir", workspaceDir);
					userProps.put("eclipseDir", Platform.getInstallLocation()
							.getURL().getFile());
					userProps.put("commonProjectName", commonProjectName);
					userProps.put("ecosystemName", ecosystemName);
					userProps.put("commonProjectPkg", commonProjectName);
					userProps.put("repoProjectURI", repoProject
							.getLocationURI().toString());
					userProps.put("pluginURI", "http://cinco.org/"
							+ ecosystemName.toLowerCase());
					userProps.put("repositoryName", repo.getName());
					userProps.put("pkgPrefix", pkgPrefix);
					userProps.put("repositoryPkg", repoProjectName);

					IFile repoPropsFile = repoProject
							.getFile("repository.properties");
					java.util.Properties props = new java.util.Properties();
					props.putAll(userProps);
					FileOutputStream propOS = new FileOutputStream(
							repoPropsFile.getLocation().toOSString());
					props.store(propOS,
							"Configuration properties for " + repo.getName());

					// Set project dependencies
					IProjectDescription desc = repoProject.getDescription();
					desc.setReferencedProjects(new IProject[] { commonProject });
					repoProject.setDescription(desc, new NullProgressMonitor());

					IFolder binFolder = repoProject.getFolder("bin");
					if (!binFolder.exists()) {
						binFolder.create(false, true, new SubProgressMonitor(
								monitor, 5));
					}

					// Create the 'src' folder
					IFolder sourceFolder = repoProject.getFolder("src");
					if (!sourceFolder.exists()) {
						sourceFolder.create(false, true,
								new SubProgressMonitor(monitor, 5));
					}

					ArrayList<String> reqBundleNames = new ArrayList<String>();
					reqBundleNames.add(commonProjectName);
					GeneratorHelper.createProjectStructure(monitor,
							repoProject, repoProjectName, true, reqBundleNames);
				}
			} catch (IOException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			} catch (CoreException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			}
		}

		// Check if there are any repositories in the model!
		if (repoProjects.size() < 1) {
			return Status.CANCEL_STATUS;
		}

		/**
		 * Task 3: Launch SEAM2PSM M2M transformations through the build.xml Ant
		 * scripts generated in the repositories.
		 */
		int monInc = 1000 / (repoProjects.size());
		ArrayList<IProject> commonProjectList = new ArrayList<IProject>();
		commonProjectList.add(commonProject);

		try {
			// 3.1: Generate the models itemtype and repository ecore models
			runAntforProjects(monitor, repoProjects, monInc / 3, "SEAM2EMF");

			// 3.2: Update references for the ecore models
			runAntforProjects(monitor, commonProjectList, monInc / 3,
					"SetCommonReferences");
			runAntforProjects(monitor, repoProjects, monInc / 3,
					"SetReferences");
/** 
 * Since EMF default code generator and the corresponding genmodels do not support cyclic dependencies (between 
 * distinct genmodels) we now generate a single genmodel under the common project which composes ALL generated ecore models!
 */
			// 3.3: Create shared genmodel
			runAntforProjects(monitor, commonProjectList, 10, "CreateGenModel");
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"RefineSharedGenModelwithItemtypeModel");
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"RefineSharedGenModelwithRepositoryModel");
			
			// 3.4: Create the OSGi descriptor
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"SEAM2OSGi");

			/* OLD STUFF: Generating individual genmodels for individual projects.
			// 3.3: Crete genmodels
			runAntforProjects(monitor, commonProjectList, 10, "CreateGenModel");
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"CreateGenModel");

			// 3.4: Update references for the genmodels
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"RefineGenModel");
			runAntforProjects(monitor, commonProjectList, 10, "RefineGenModel");

			// 3.5: Create repository genmodels
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"CreateRepoGenModel");
			runAntforProjects(monitor, repoProjects, monInc / 2,
					"RefineRepoGenModel");
*/
		} catch (CoreException e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}

		/**
		 * Task 4: GenModel code generation
		 */
/*
		try {

	
			ResourceSet rSet = new ResourceSetImpl();
			rSet.getPackageRegistry().put(EcorePackage.eNS_URI,
					EcorePackage.eINSTANCE);
			rSet.getPackageRegistry().put(GenModelPackage.eNS_URI,
					GenModelPackage.eINSTANCE);
			rSet.getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());

			// 4.1: Create project URI map from platform:/resource to file:/ URIs
			// Create URIMap for URIConverter
			List<EPackage> pkgList = new ArrayList<EPackage>();
			for(IProject project : repoProjects) {
				rSet.getPackageRegistry().put(EcorePackage.eNS_URI,
						EcorePackage.eINSTANCE);
				IFile itFile = project.getFile("model/itemtype.ecore");
				
				URI platURI = URI.createPlatformResourceURI(itFile.getFullPath()
						.toString(), true);
				URI fileURI = URI.createFileURI(itFile.getFullPath().toOSString());
				Resource res = rSet.getResource(platURI, true);
				if(res.getContents().get(0) instanceof EPackage) {
					EPackage pkg = (EPackage) res.getContents().get(0);
					pkgList.add(pkg);
					rSet.getURIConverter().getURIMap().put(platURI, fileURI);
				} else {
					System.out.println("Invalid itemtype.ecore " + platURI);
					return Status.CANCEL_STATUS;
				}
			}
			
			// 4.1: Generate common Java classes
			IFile commonGenFile = commonProject.getFolder("model").getFile(
					"common.genmodel");
			IFile commonEcoreFile = commonProject.getFolder("model").getFile("common.ecore");
			
			URI uri = URI.createURI(commonGenFile.getLocationURI().toString());
			Resource commonGenRes = rSet.getResource(uri, true);

			URI ecoreUri = URI.createURI(commonEcoreFile.getLocationURI().toString());
			Resource commonEcoreRes = rSet.getResource(ecoreUri, true);
			
			pkgList.add((EPackage) commonEcoreRes.getContents().get(0));
			
			if (commonGenRes.getContents().get(0) instanceof GenModel) {
				GenModel genModel = (GenModel) commonGenRes.getContents()
						.get(0);
				System.out.println("Generating Java from genmodel " + uri);

				genModel.initialize(pkgList);
				
				generateJava(genModel);
			} else {
				System.err.println("No genmodel found in uri " + uri);
				return Status.CANCEL_STATUS;
			}
*/
//			runAntforProjects(monitor, commonProjectList, 10, "common2Java");

			/**
			 * Tasks 4.2 & 4.3 are not applicable, since we have only one single genmodel for now!
			 */
			// 4.2: Generate itemtype Java classes
/*			for (IProject project : repoProjects) {
				IFile genFile = project.getFolder("model").getFile(
						"itemtype.genmodel");
				URI genURI = URI.createURI(genFile.getLocationURI().toString());
				Resource genRes = rSet.getResource(genURI, true);
				if (genRes.getContents().get(0) instanceof GenModel) {
					GenModel genModel = (GenModel) genRes.getContents().get(0);
					System.out.println("Generating Java from genmodel "
							+ genURI);
					generateJava(genModel);
				} else {
					System.err.println("No genmodel found in uri " + genURI);
					return Status.CANCEL_STATUS;
				}
			}
			// runAntforProjects(monitor, repoProjects, 10, "itemtype2Java");

			// 4.3: Generate repository Java classes
			for (IProject project : repoProjects) {
				IFile genFile = project.getFolder("model").getFile(
						"repository.genmodel");
				URI genURI = URI.createURI(genFile.getLocationURI().toString());
				Resource genRes = rSet.getResource(genURI, true);
				if (genRes.getContents().get(0) instanceof GenModel) {
					GenModel genModel = (GenModel) genRes.getContents().get(0);
					System.out.println("Generating Java from genmodel "
							+ genURI);
					generateJava(genModel);
				} else {
					System.err.println("No genmodel found in uri " + genURI);
					return Status.CANCEL_STATUS;
				}
			}
			// runAntforProjects(monitor, repoProjects, 10, "repository2Java");

		} finally {
			System.out.println("Genmodel Java generated.");
		}
*/
		/*
		 * catch (CoreException e) { e.printStackTrace(); return
		 * Status.CANCEL_STATUS;
		 * 
		 * } catch (InterruptedException e) { e.printStackTrace(); return
		 * Status.CANCEL_STATUS; }
		 */
		
		return Status.OK_STATUS;
	}

	private void generateJava(GenModel genModel) {
		GeneratorAdapterFactory.Descriptor.Registry.INSTANCE.addDescriptor(
				GenModelPackage.eNS_URI,
				GenModelGeneratorAdapterFactory.DESCRIPTOR);

		// Create the generator and set the model-level input object.
		//
		Generator generator = new Generator();
		generator.setInput(genModel);
		// Generator model code.
		//
		genModel.setForceOverwrite(true);
		genModel.reconcile();
		genModel.setCanGenerate(true);
		genModel.setValidateModel(true);
		genModel.setUpdateClasspath(false);
		genModel.setCodeFormatting(true);
		generator.generate(genModel,
				GenBaseGeneratorAdapter.MODEL_PROJECT_TYPE,
				new BasicMonitor.Printing(System.out));
	}

	private void runAntforProjects(IProgressMonitor monitor,
			List<IProject> repoProjects, int monInc, String executionTarget)
			throws CoreException, InterruptedException {
		for (IProject repoProject : repoProjects) {
			System.out.println("Run Ant target " + executionTarget
					+ " for project " + repoProject.getName());
			IFile antFile = repoProject.getFile("build.xml");
			AntRunner runner = new AntRunner();

			// runner.addBuildLogger(className);
			runner.setBuildFileLocation(antFile.getLocation().toOSString());
			runner.setArguments("-Dmessage=Building -verbose");
			// runner.addUserProperties(userProps);
			runner.setExecutionTargets(new String[] { executionTarget });
			
			runner.run(new SubProgressMonitor(monitor, 10));
			while (AntRunner.isBuildRunning()) {
				this.wait(50);
			}
			monitor.worked(monInc);
			repoProject.refreshLocal(IResource.DEPTH_INFINITE, null);
		}

		while (AntRunner.isBuildRunning()) {
			try {
				this.wait(50);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	private void createFile(String fileName, Bundle bundle, IProject repoProject)
			throws IOException, CoreException {
		Path buildPropsPath = new Path(fileName);
		InputStream buildPropsStream = FileLocator.openStream(bundle,
				buildPropsPath, false);
		IFile buildPropFile = repoProject.getFile(fileName);
		if (!buildPropFile.exists()) {
			buildPropFile.create(buildPropsStream, true,
					new NullProgressMonitor());
		}
	}

}
