package org.cinco.generator.plugin.jobs;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.cinco.generator.plugin.RepositoryGeneratorConstants;
import org.cinco.generator.plugin.preferences.PreferenceConstants;
import org.cinco.metamodel.ecosystem.seadescription.SEADescription;
import org.cinco.metamodel.ecosystem.seadescription.SeadescriptionPackage;
import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.preferences.IPreferencesService;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UML212UMLResource;
import org.osgi.framework.Bundle;

public class UML2SEAMJob extends Job {

	private String pkgPrefix;
	private IFile umlFile;
	private Resource umlResource;
	private String ecosystemName;
	private String commonProjectName;
	private IProject commonProject;

	public static final String SEAM_POSTFIX = "xmi";
	
	public UML2SEAMJob(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		// Read the properties common for all jobs.
		monitor.beginTask("Create Ecosystem Facilities", 1000);

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		String workspaceDir= workspace.getRoot().getLocation().toFile().getPath().toString(); 
		String eclipseDir = System.getProperty("eclipse.launcher");
		System.out.println("Eclipse directory: " + eclipseDir);
		System.out.println("Workspace directory: " + workspaceDir);
		
		IPreferencesService prefService = Platform.getPreferencesService();
		pkgPrefix = prefService.getString("org.cinco.generator.plugin",
				PreferenceConstants.P_PKGPREFIX, null, null);
		if(pkgPrefix == null) {
			pkgPrefix = "org.cinco";
		}
		// The pkgPrefix is assumed in the following code to end
		// with a dot ".".
		if ((pkgPrefix.length() > 1) && (!pkgPrefix.endsWith("."))) {
			pkgPrefix = pkgPrefix + ".";
		}

		/**
		 * Task 1: Generate the SEAM model from the UML model.
		 */
		// 1.1: Read UML file
		ResourceSet umlResourceSet = new ResourceSetImpl();
		umlResourceSet.getPackageRegistry().put(UMLPackage.eNS_URI,
				UMLPackage.eINSTANCE);
		umlResourceSet.getPackageRegistry().put(
				UML212UMLResource.UML_METAMODEL_NS_URI, UMLPackage.eINSTANCE);

		QualifiedName umlFilekey = new QualifiedName(null,
				RepositoryGeneratorConstants.UML_IFILE);
		umlFile = (IFile) this.getProperty(umlFilekey);

		URI umlURI = URI.createPlatformResourceURI(umlFile.getFullPath()
				.toString(), true);

		umlResource = umlResourceSet.getResource(umlURI, true);
		if (umlResource == null) {
			System.err.println("No UML file found from " + umlURI);
			return Status.CANCEL_STATUS;
		}
		;
		System.out.println("Loaded UML model from URI " + umlResource.getURI());

		if (!(umlResource.getContents().get(0) instanceof Model)) {
			System.err.println("Not a UML Model! " + umlResource.getURI());
			return Status.CANCEL_STATUS;
		}
		Model umlModel = (Model) umlResource.getContents().get(0);

		// Try to locate the ecosystem model UML package
		if(umlModel.getAppliedStereotype("ecosystem::s_ecosystemmodel") != null) {
			ecosystemName = umlModel.getName();
		} else {
			for (org.eclipse.uml2.uml.Package p : umlModel.getNestedPackages()) {
				System.out.println("Inspecting package " + p.getName());
				Stereotype st = p
						.getAppliedStereotype("ecosystem::s_ecosystemmodel");
				if (st != null) {
					ecosystemName = p.getName();
					break;
				}
			}
		}
		
		if (ecosystemName == null) {
			System.err.println("No ecosystem model found from model in URI "
					+ umlResource.getURI());
			return Status.CANCEL_STATUS;
		}
		
		System.out.println("Ecosystem name = " + ecosystemName);

		// Task 1.2: Create an eclipse project for the common parts of ecosystem facilities
		commonProjectName = pkgPrefix + ecosystemName.toLowerCase() + ".common";
		commonProject = workspace.getRoot().getProject(commonProjectName);
		IFile seamFile = commonProject.getFile("model/" + ecosystemName + "." + SEAM_POSTFIX);
		if (!seamFile.exists()) {
			try {
				if (!commonProject.exists()) {
					commonProject.create(new SubProgressMonitor(monitor, 5));
				}
				commonProject.open(new SubProgressMonitor(monitor, 5));
				
				GeneratorHelper.createProjectStructure(new SubProgressMonitor(monitor, 5), commonProject, commonProjectName, false, null);

				// First create the build.xml file to the common project which
				// "does the job"
				Bundle bundle = Platform.getBundle("org.cinco.generator.plugin");
				Path path = new Path("scripts/build-common.xml");
				InputStream buildStream = FileLocator.openStream(bundle, path,
						false);
				IFile antFile = commonProject.getFile("build.xml");
				if (!antFile.exists()) {
					antFile.create(buildStream, true, new SubProgressMonitor(
							monitor, 5));
				}

				String pkgPath = "src/" + (commonProjectName.replace('.', '/'));
				IFolder targetFolder = commonProject.getFolder(pkgPath);

				/**
				 * Create a lauch configuration for the AntRunner.
				 */ 
				Map<String, String> userProps = new HashMap<String, String>();
				userProps.put("workspaceDir", workspaceDir);
				userProps.put("eclipseDir", eclipseDir);
				userProps.put("ecosystemName", ecosystemName);
				userProps.put("commonProjectURI", commonProject.getLocationURI()
						.toString());
				userProps.put("commonProjectPkg", commonProjectName);
				userProps.put("pluginURI", "http://cinco.org/" + ecosystemName.toLowerCase());
				userProps.put("umlModelURI", umlFile.getLocationURI().toString()); // umlFile.getLocationURI().toString());
				userProps.put("pkgPrefix", pkgPrefix);
				System.out.println(userProps);
				IFile repoPropsFile = commonProject
						.getFile("repository.properties");
				java.util.Properties props = new java.util.Properties();
				props.putAll(userProps);
				FileOutputStream propOS = new FileOutputStream(
						repoPropsFile.getLocation().toOSString());
				props.store(propOS,
						"Configuration properties for " + ecosystemName + " repositories.");
				
				AntRunner runner = new AntRunner();
				runner.setBuildFileLocation(antFile.getLocation().toOSString());
				runner.setArguments("-Dmessage=Building -verbose");
				runner.addUserProperties(userProps);
				runner.setExecutionTargets(new String[] { "all" });
				runner.run(new SubProgressMonitor(monitor, 100));


				/*
				 * Task 2: Now generate the ecosystem specific ant build file for generating all the repository implementations.
				 */
				while(AntRunner.isBuildRunning()) {
					System.out.println("Waiting for AntRunner..");
					this.wait(100);
				}
				monitor.worked(100);
				
				// 2.1: Read the generated SEAM model.
				Resource modelRes = null;
				ResourceSet rSet = new ResourceSetImpl();
				rSet.getPackageRegistry().put(
						SeadescriptionPackage.eNS_URI,
						SeadescriptionPackage.eINSTANCE);
				URI seamURI = URI.createPlatformResourceURI(seamFile.getFullPath()
						.toString(), true);
				modelRes = rSet.getResource(seamURI, true);

				if (!(modelRes.getContents().get(0) instanceof SEADescription)) {
					System.err.println("No SEADescription element found from "
							+ modelRes.getURI());
					return Status.CANCEL_STATUS;
				}
				monitor.worked(100); // Read the KnowledgeManagementModel :)

				// 2.2: Generate the Ant build file.
				EObject seamObject = modelRes.getContents().get(0);
				IFile buildFile = commonProject.getFile("build-repositories.xml");

				System.out.println("Target file for build-repositories.xml: "
						+ buildFile.getLocation().toOSString());

				org.cinco.m2t.seam2ant.main.SEAM2Ant gen0 =
				new org.cinco.m2t.seam2ant.main.SEAM2Ant(seamObject,
						commonProject.getLocation().toFile(),
						new ArrayList<String>());
				String generationID = org.eclipse.acceleo.engine.utils.AcceleoLaunchingUtil
						.computeUIProjectID(
								"org.cinco.generator.m2t",
								"org.cinco.generator.m2t.java.main.Generate",
								seamURI.toString(), commonProject.getLocation().toOSString(),
								new ArrayList<String>());
				gen0.setGenerationID(generationID);
				gen0.addPropertiesFile(repoPropsFile.getLocation()
						.toOSString());
				gen0.doGenerate(BasicMonitor.toMonitor(monitor));
				monitor.worked(100);
				
			} catch (IOException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			} catch (CoreException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			} catch (InterruptedException e) {
				e.printStackTrace();
				return Status.CANCEL_STATUS;
			}
		}

		return Status.OK_STATUS;
	}

}
