package org.cinco.qvt.lib;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.codegen.ecore.genmodel.GenClass;
import org.eclipse.emf.codegen.ecore.genmodel.GenFeature;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.BasicFeatureMap;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.ecore.util.FeatureMapUtil;
import org.eclipse.m2m.qvt.oml.blackbox.java.Operation;
import org.eclipse.soa.sca.sca1_1.model.sca.Component;
import org.eclipse.soa.sca.sca1_1.model.sca.Implementation;
import org.eclipse.soa.sca.sca1_1.runtime.tuscany.model.tuscany2.OSGiImplementation;
import org.eclipse.soa.sca.sca1_1.runtime.tuscany.model.tuscany2.OSGiProperty;
import org.eclipse.soa.sca.sca1_1.runtime.tuscany.model.tuscany2.Tuscany2Factory;
import org.eclipse.soa.sca.sca1_1.runtime.tuscany.model.tuscany2.Tuscany2Package;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Stereotype;

public class UtilitiesLibrary {

	public UtilitiesLibrary() {
		super();
	}

	@Operation(contextual=true)
	public static void setValue(EObject self, String propertyName, Object value) {
		EClass cls = self.eClass();
		EStructuralFeature feat = cls.getEStructuralFeature(propertyName);
		EClassifier fType = feat.getEType();
		if(feat != null) {
			self.eSet(feat, value);
		}
	}

	@Operation(contextual=true)
	public static void setCollectionValue(EObject self, String propertyName, List<Object> value) {
		EClass cls = self.eClass();
		EStructuralFeature feat = cls.getEStructuralFeature(propertyName);
		EClassifier fType = feat.getEType();
		if(feat != null) {
			self.eSet(feat, value);
		}
	}
	
	@Operation(contextual=true)
	public static void setCollectionValue(Element self, Stereotype st, String propertyName, List<Object> value) {
		self.setValue(st, propertyName, value);
	}
	
	@Operation(contextual=true)
	public static Object getElementValue(Element self, Stereotype st, String propertyName) {
		return self.getValue(st, propertyName);
	}

	@Operation(contextual=true)
	public static Object getElementValue(EObject self, String propertyName) {
		EClass cls = self.eClass();
		EStructuralFeature feat = cls.getEStructuralFeature(propertyName);
		return self.eGet(feat);
	}
	
	@SuppressWarnings("unchecked")
	@Operation(contextual=true)
	public static List<Object> getElementCollectionValue(Element self, Stereotype st, String propertyName) {
		return (List<Object>) self.getValue(st, propertyName);
	}
	
	@Operation(contextual=false)
	public static EObject getEObjectforURI(String uri) {
		URI myURI = URI.createURI(uri);
		EObject result = null;
		ResourceSet resourceSet = new ResourceSetImpl();
		try {
			result = resourceSet.getEObject(myURI, true);
		} catch (Exception e) {
			System.err.println("Exception " + e.getMessage());
		}
		return result;
	}

	@Operation(contextual=true)
	public static String getResourceURI(EObject self) {
		System.out.println("getResourceURI eResource " + self.eResource().toString());
		System.out.println("eResource.getURI " + self.eResource().getURI());
		return self.eResource().getURI().toString();
	}
	
	@Operation(contextual=true)
	public static String getEObjectURI(EObject self) {
		String frag = self.eResource().getURIFragment(self);
		return EcoreUtil.getURI(self).toString() + "#//" + frag;
	}

	@Operation(contextual=false)
	public static EObject getEcoreGenPackage() {
		System.out.println("getEcoreGenPackage()");
		ResourceSet resSet = new ResourceSetImpl();
		resSet.getPackageRegistry().put(GenModelPackage.eNS_URI, GenModelPackage.eINSTANCE);
		URI uri = URI.createPlatformPluginURI("/org.eclipse.emf.ecore/model/Ecore.genmodel", true);
		Resource res = resSet.getResource(uri, true);
		if(res == null) {
			System.err.println("Couldn't load genModel with URI " + uri.toString());
			return null;
		}
		GenModel genModel = (GenModel) res.getContents().get(0);
		System.out.println("resource " + res.toString());
		System.out.println("genPackage " + genModel.getGenPackages().get(0));
		return 	genModel.getGenPackages().get(0);
	}
	
	/**
	 * Returns the genPackage corresponding to an ECORE::EPackage
	 * @param self
	 * @return
	 */
	@Operation(contextual=true)
	public static EObject getGenPackage(EPackage self) {
		return getGenModel(self).getGenPackages().get(0);
	}

	/**
	 * Returns the GenModel corresponding to an EObject
	 * @param self
	 * @return
	 */
	private static GenModel getGenModel(EObject self) {
		String resURI = getResourceURI(self);
		String genModelURI = resURI.replaceFirst(".ecore", ".genmodel");
		
		ResourceSet gmResourceSet = new ResourceSetImpl();
		gmResourceSet.getPackageRegistry().put(GenModelPackage.eNS_URI,
				GenModelPackage.eINSTANCE);
//		URI uri = URI.createPlatformResourceURI(genModelURI, true);
		URI uri = URI.createURI(genModelURI);
		System.out.println("getGenModel URI " + uri);
		Resource res = gmResourceSet.getResource(uri, true);
		if(res == null) {
			System.err.println("Couldn't load genModel with URI " + uri.toString());
			return null;
		}
		return (GenModel) res.getContents().get(0);
	}

	/**
	 * Returns external dependencies (usedGenPackages) for a GenModel element.
	 * @param self An element of kind GenModel
	 * @return GenPackage references
	 */
	@Operation(contextual=true)
	public static List<EObject> getGenModelDependencies(EObject self) {
		List<EObject> res = new ArrayList<EObject>();

		if(self instanceof GenModel) {
			GenModel m = (GenModel) self;
			List<String> visited = new ArrayList<String>();
			
			visited.add(self.eResource().getURI().toString());	// Do not add "self" to the usedGenPackages!

			List<GenPackage> pkgs = m.getGenPackages();
			List<GenPackage> allPkgs = new ArrayList<GenPackage>();
			for(GenPackage p : pkgs) {
				allPkgs.add(p);
				allPkgs.addAll(p.getNestedGenPackages());
			}

			for(GenPackage p : allPkgs) {

				// Get dependencies induced by references
				for(GenFeature f : p.getAllGenFeatures()) {
					EStructuralFeature feat = f.getEcoreFeature();
					if(feat instanceof EReference) {
						if(!feat.getEType().eResource().getURI().isPlatformPlugin()) {
							GenModel typeGenModel = getGenModel(feat.getEType());
							if((typeGenModel != null) 
									&& (typeGenModel.eResource().getURI().toString() != m.eResource().getURI().toString())
									) {
								System.out.println("getGenModelDependencies: different uris " +
										typeGenModel.eResource().getURI().toString() + "<-->" +
										m.eResource().getURI().toString());
								res.addAll(getGenModelDependencies(typeGenModel, visited));
							}
						}
					}
				}
				
				// Get dependencies induced by generalization hierarchies
				for(GenClass c : p.getGenClasses()) {
					System.out.println("class to examine: " + c.getEcoreClass().getName());
					EClass myClass = c.getEcoreClass();
					for(EClass sup : myClass.getESuperTypes()) {
						System.out.println("has supertype " + sup.getName());
						GenModel supGenModel = getGenModel(sup);
						assert(supGenModel != null);
						if(supGenModel.eResource().getURI().toString() != m.eResource().getURI().toString()) {
							System.out.println("Supertype is in another genModel!");
							res.addAll(getGenModelDependencies(supGenModel, visited));
						}
					}
				}
			}

		}
		return res;
	}

	private static List<EObject> getGenModelDependencies(EObject self, List<String> visited) {
		System.out.println("getGenModelDependencies " + self.eResource().getURI().toString());
		System.out.println("                visited " + visited);

		List<EObject> candidates = new ArrayList<EObject>();
		
		if(self instanceof GenModel) {
			GenModel m = (GenModel) self;

			if(visited.contains(self.eResource().getURI().toString())) {
				System.out.println("already visited");
				return new ArrayList<EObject>();
			}
			
			visited.add(m.eResource().getURI().toString());
			
			List<GenPackage> pkgs = m.getGenPackages();
			List<GenPackage> allPkgs = new ArrayList<GenPackage>();
			for(GenPackage p : pkgs) {
				allPkgs.add(p);
				allPkgs.addAll(p.getNestedGenPackages());
			}
			
			candidates.addAll(allPkgs);
			
			// Recursion to all external packages
			for(GenPackage p : allPkgs) {
				
				// Get dependencies induced by references
				for(GenFeature f : p.getAllGenFeatures()) {
				EStructuralFeature feat = f.getEcoreFeature();
					if(feat instanceof EReference) {
						System.out.println("reference to examine: " + feat.getName());
						GenModel typeGenModel = getGenModel(feat.getEType());
						assert(typeGenModel != null);
						
						// Is this an external genModel? (not "self" given as a parameter)
						if((typeGenModel != null) 
								&& (typeGenModel.eResource().getURI().toString() != m.eResource().getURI().toString())
								) {
							System.out.println("getGenModelDependencies: different uris " +
									typeGenModel.eResource().getURI().toString() + "<-->" +
									m.eResource().getURI().toString());
							candidates.addAll(getGenModelDependencies(typeGenModel, visited));
						}
					}
				}
				
				// Get dependencies induced by generalization hierarchies
				for(GenClass c : p.getGenClasses()) {
					System.out.println("class to examine: " + c.getEcoreClass().getName());
					EClass myClass = c.getEcoreClass();
					for(EClass sup : myClass.getESuperTypes()) {
						System.out.println("has supertype " + sup.getName());
						GenModel supGenModel = getGenModel(sup);
						assert(supGenModel != null);
						if(supGenModel.eResource().getURI().toString() != m.eResource().getURI().toString()) {
							System.out.println("Supertype is in another genModel!");
							candidates.addAll(getGenModelDependencies(supGenModel, visited));
						}
					}
				}
			}
		}
		
		// Create a SET of root GenPackages (getRootGenPackage is reflexive operation)
		Map<String, EObject> resHash = new HashMap<String, EObject>();
		for(EObject o : candidates) {
			GenPackage op = (GenPackage) o;
			resHash.put(op.getRootGenPackage().getEcorePackage().getNsURI(), op.getRootGenPackage());
		}

		List<EObject> res = new ArrayList<EObject>();
		res.addAll(resHash.values());
		return res;
	}
	
	@Operation(contextual=true)
	public static List<EObject> getGenPackageDependencies(EObject self) {
		System.out.println("getGenPackageDependencies for " + self);
		List<EObject> res = new ArrayList<EObject>();
		if(self instanceof GenPackage) {
			GenPackage pkg = (GenPackage) self;
			for(GenPackage d : pkg.getPackageInterDependencies()) {
				System.out.println("inter-dependency:" + d);
				res.add(d);
			}
		}
		return res;
	}
	
	@Operation(contextual=true)
	public static FeatureMap setMixedValue(EObject self, String featName, EObject value) {
		EClass cls = self.eClass();
		EStructuralFeature feat = cls.getEStructuralFeature(featName);
		if(feat == null) {
			return null;
		}
		FeatureMap res = (FeatureMap) self.eGet(feat, true);
		if(res == null) {
			System.out.println("res is null");
			res = new BasicFeatureMap((InternalEObject) self, feat.getFeatureID());
		}
		res.add(FeatureMapUtil.createEntry(feat, value));
		
		return res;
	}
	
	@Operation(contextual=true)
	public static EStructuralFeature getFeature(FeatureMap.Entry self) {
		return self.getEStructuralFeature();
	}
	
	@Operation(contextual=true)
	public static FeatureMap.Entry setValue(FeatureMap.Entry self, Object value) {		
		EStructuralFeature feat = self.getEStructuralFeature();
		System.err.println("Structural feature: " + feat.toString());
		return FeatureMapUtil.createEntry(feat, value);
	}
	
	@Operation(contextual = true)
	public static Object getValue(FeatureMap.Entry self)
	{
	  return self.getValue();
	}
	
	/**
	 * SCA -specific getters / setters for "mixed" values
	 */
	
	@Operation(contextual=true)
	public static EObject setComponentImplementation(EObject self, EObject impl) throws Exception {
		if((Component.class.isInstance(self)) && (Implementation.class.isInstance(impl))) {
			Component c = (Component) self;
			c.getAny().add(Tuscany2Package.eINSTANCE.getDocumentRoot_ImplementationOsgi(), impl);
		} else {
			throw new Exception("Invalid parameters: self = " + 
					self.toString() + " is not instance of " + (Component.class) 
					+ ", impl = " + 
					impl.toString() + " is not instance of " + (Implementation.class)
			);
		}
		return self;
	}

	/**
	 * 
	 * @param self TUSCANY::OSGiImplementation
	 * @param propName name of the property e.g. "service"
	 * @param value value of the property
	 * @return
	 */
	@Operation(contextual=true)
	public static EObject setOSGiImplementationProperties(EObject self, String propName, String value) {
		OSGiImplementation impl = (OSGiImplementation) self;
		OSGiProperty prop = Tuscany2Factory.eINSTANCE.createOSGiProperty();
		prop.setName(propName);
		prop.setValue(value);
		impl.getAny().add(Tuscany2Package.eINSTANCE.getDocumentRoot_OsgiProperty(), prop);
		return self;
	}
}
